@@ -40,7 +40,7 @@ class GnomeDialog(_gtk.GtkWindow):
 	def button_connect(self, button, callback):
 		_gnomeui.gnome_dialog_button_connect(self._o, button, callback)
 	def set_default(self, button):
-		_gnomeui.gnome_dialog_set_default(self._o)
+		_gnomeui.gnome_dialog_set_default(self._o, default)
 	def set_sensitive(self, button, setting):
 		_gnomeui.gnome_dialog_set_sensitive(self._o, button, setting)
 	def set_destroy(self, self_destruct):
@@ -328,8 +328,9 @@ class GnomeCanvas(_gtk.GtkLayout):
 	def update_now(self):
 		_gnomeui.gnome_canvas_update_now(self._o)
 	def get_item_at(self, x, y):
-		return _obj2inst(_gnomeui.gnome_canvas_get_item_at(self._o,
-								   x, y))
+		ret = _gnomeui.gnome_canvas_get_item_at(self._o, x, y)
+		if (ret): return _obj2inst(ret)
+		return None
 	def request_redraw(self, x1,y1, x2,y2):
 		_gnomeui.gnome_canvas_request_redraw(self._o, x1,y1, x2,y2)
 	def w2c(self, wx, wy):
