@@ -125,6 +125,8 @@ static int (*PyGtkEnum_get_value)(GtkType type, PyObject *obj, gint *val)=0;
 static int (*PyGtkFlag_get_value)(GtkType type, PyObject *obj, gint *val)=0;
 
 static GtkArg *(*PyDict_AsGtkArgs)(PyObject *dict, GtkType type, gint*nargs)=0;
+static void (*PyGtk_BlockThreads)(void);
+static void (*PyGtk_UnblockThreads)(void);
 
 /* --- gnomeui stuff --- */
 
@@ -547,6 +549,21 @@ static PyObject *_wrap_gnome_canvas_get_color(PyObject *self, PyObject *args) {
   return NULL;
 } 
 
+static PyObject *_wrap_gnome_canvas_get_item_at(PyObject *self, PyObject *args) {
+    double x, y;
+    PyObject *canvas;
+    GnomeCanvasItem *item;
+
+    if (!PyArg_ParseTuple(args, "O!dd:gnome_canvas_get_item_at", PyGtk_Type,
+			  &canvas, &x, &y))
+        return NULL;
+    item = gnome_canvas_get_item_at(GNOME_CANVAS(PyGtk_Get(canvas)), x, y);
+    if (item)
+        return PyGtk_New((GtkObject *)item);
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 static PyObject *_wrap_gnome_canvas_item_new(PyObject *self, PyObject *args) {
   PyObject *parent, *dict, *ret;
   GtkType type;
@@ -603,6 +620,7 @@ static PyObject *_wrap_gnome_canvas_item_affine_relative(PyObject *self, PyObjec
       PyErr_SetString(PyExc_TypeError, "sequence item not a float");
       return NULL;
     }
+    Py_DECREF(sitem);
   }
   gnome_canvas_item_affine_relative(GNOME_CANVAS_ITEM(PyGtk_Get(item)),affine);
   Py_INCREF(Py_None);
@@ -630,6 +648,7 @@ static PyObject *_wrap_gnome_canvas_item_affine_absolute(PyObject *self, PyObjec
       PyErr_SetString(PyExc_TypeError, "sequence item not a float");
       return NULL;
     }
+    Py_DECREF(sitem);
   }
   gnome_canvas_item_affine_absolute(GNOME_CANVAS_ITEM(PyGtk_Get(item)),affine);
   Py_INCREF(Py_None);
@@ -744,6 +763,7 @@ static void PyGnome_interact_function(gpointer a, PyObject *func, int nargs,
 			      PyGtk_New(GTK_VALUE_OBJECT(args[0])),
 			      GTK_VALUE_INT(args[1]),
 			      GTK_VALUE_ENUM(args[2])); */
+  PyGtk_BlockThreads();
   ret = PyObject_CallFunction(func, "(Oii)",
 			      PyGtk_New(*GTK_RETLOC_OBJECT(args[1])),
 			      *GTK_RETLOC_INT(args[2]),
@@ -755,9 +775,11 @@ static void PyGnome_interact_function(gpointer a, PyObject *func, int nargs,
         PyErr_Print();
         PyErr_Clear();
     }
+    PyGtk_UnblockThreads();
     return;
   }
   Py_DECREF(ret);
+  PyGtk_UnblockThreads();
 }
 
 static PyObject *_wrap_gnome_client_request_interaction(PyObject *self,
@@ -1144,6 +1166,7 @@ static GtkWidget *PyGtk_cauldron_callback(GtkWidget *self, PyObject *tuple) {
   PyObject *ret, *params;
   GtkWidget *wret;
 
+  PyGtk_BlockThreads();
   ret = PyTuple_New(1);
   PyTuple_SetItem(ret, 0, PyGtk_New((GtkObject *)self));
   params = PySequence_Concat(ret, PyTuple_GetItem(tuple, 1));
@@ -1158,26 +1181,31 @@ static GtkWidget *PyGtk_cauldron_callback(GtkWidget *self, PyObject *tuple) {
         PyErr_Print();
         PyErr_Clear();
     }
+    PyGtk_UnblockThreads();
     return NULL;
   }
 
   if (ret == Py_None) { /* this should only be returned by 'c' type callbacks*/
     Py_DECREF(ret);
+    PyGtk_UnblockThreads();
     return NULL;
   } else if (PyGtk_Check(ret)) {
     wret = GTK_WIDGET(PyGtk_Get(ret));
     /* the returned widget should still have its floating reference set */
     Py_DECREF(ret);
+    PyGtk_UnblockThreads();
     return wret;
   } else if ((params = PyObject_GetAttrString(ret, "_o")) &&
 	     PyGtk_Check(params)) { /* for Gtkinter */
     wret = GTK_WIDGET(PyGtk_Get(params));
     Py_DECREF(params);
     Py_DECREF(ret);
+    PyGtk_UnblockThreads();
     return wret;
   }
   Py_XDECREF(params);
   Py_DECREF(ret);
+  PyGtk_UnblockThreads();
   return NULL;
 }
 
@@ -1317,6 +1345,7 @@ static PyObject *_wrap_gtk_dialog_cauldron(PyObject *self, PyObject *args) {
 static void PyGnome_StringCallback(gchar *string, gpointer data) {
   PyObject *ret, *func = (PyObject *)data;
 
+  PyGtk_BlockThreads();
   ret = PyObject_CallFunction(func, "s", string);
   if (ret)
     Py_DECREF(ret);
@@ -1329,11 +1358,13 @@ static void PyGnome_StringCallback(gchar *string, gpointer data) {
     }
   }
   Py_DECREF(func); /* callback only called once */
+  PyGtk_UnblockThreads();
 }
 
 static void PyGnome_ReplyCallback(gint reply, gpointer data) {
   PyObject *ret, *func = (PyObject *)data;
 
+  PyGtk_BlockThreads();
   ret = PyObject_CallFunction(func, "i", reply);
   if (ret)
     Py_DECREF(ret);
@@ -1346,6 +1377,7 @@ static void PyGnome_ReplyCallback(gint reply, gpointer data) {
     }
   }
   Py_DECREF(func); /* callback only called once */
+  PyGtk_UnblockThreads();
 }
 
 static PyObject *_wrap_gnome_question_dialog(PyObject *self, PyObject *args) {
@@ -1530,6 +1562,7 @@ static PyMethodDef gnomeuiMethods[] = {
     { "gnome_canvas_w2c", _wrap_gnome_canvas_w2c, 1 },
     { "gnome_canvas_c2w", _wrap_gnome_canvas_c2w, 1 },
     { "gnome_canvas_get_color", _wrap_gnome_canvas_get_color, 1 },
+    { "gnome_canvas_get_item_at", _wrap_gnome_canvas_get_item_at, 1 },
     { "gnome_canvas_item_new", _wrap_gnome_canvas_item_new, 1 },
     { "gnome_canvas_item_set", _wrap_gnome_canvas_item_set, 1 },
     { "gnome_canvas_item_affine_relative", _wrap_gnome_canvas_item_affine_relative, 1 },
@@ -1636,6 +1669,10 @@ void init_gnomeui() {
 						"PyDict_AsGtkArgs"));
     PyGtk_RegisterBoxed = PyCObject_AsVoidPtr(PyDict_GetItemString(d,
 						"PyGtk_RegisterBoxed"));
+    PyGtk_BlockThreads = PyCObject_AsVoidPtr(
+                        PyDict_GetItemString(d, "PyGtk_BlockThreads"));
+    PyGtk_UnblockThreads = PyCObject_AsVoidPtr(
+                        PyDict_GetItemString(d, "PyGtk_UnblockThreads"));
 
     m = PyImport_ImportModule("_gdkimlib");
     if (m == NULL) {
