@@ -43,3 +43,16 @@ p.addExcludeFile('generate/gnomeui.ignore')
 p.startParsing()
 print "done"
 
+print "Outputing applet stuff ...",
+sys.stdout.flush()
+p = generate.FilteringParser(input='generate/applet.defs',
+			     prefix='appletmodule')
+p.startParsing()
+print "done"
+
+print "Outputing capplet stuff ...",
+sys.stdout.flush()
+p = generate.FilteringParser(input='generate/capplet.defs',
+			     prefix='cappletmodule')
+p.startParsing()
+print "done"
